/* This file is mostly static css that should get delivered alongside a generated
 * bento dashboard application */

/* This helps ensure no extra space is present around your content
   (Notably, a white margin around the page when using a dark theme) */
body {
	margin: 0;
}

/* By default, a loading spinner would display over a blank element, but
 * this causes discontinuity in the display between graph and other states.
 * Instead, we want the spinner to display over the stale element until loading
 * is finished and we achieve this with the following setting
 * */
*[data-dash-is-loading="true"] {
	visibility: hidden;
}

/* xxxxx Starter CSS from Plotly Dash xxxx */
/* Base Styles
–––––––––––––––––––––––––––––––––––––––––––––––––– */
/* NOTE
html is set to 62.5% so that all the REM measurements throughout Skeleton
are based on 10px sizing. So basically 1.5rem = 15px :) */
html {
	font-size: 62.5%;
}
body {
	font-size: 1.5em; /* currently ems cause chrome bug misinterpreting rems on body element */
	line-height: 1.6;
	font-weight: 400;
}

/* Typography
–––––––––––––––––––––––––––––––––––––––––––––––––– */
h1,
h2,
h3,
h4,
h5,
h6 {
	margin-top: 0;
	margin-bottom: 0;
	font-weight: 300;
}
h1 {
	font-size: 4.5rem;
	line-height: 1.2;
	letter-spacing: -0.1rem;
	margin-bottom: 2rem;
}
h2 {
	font-size: 3.6rem;
	line-height: 1.25;
	letter-spacing: -0.1rem;
	margin-bottom: 1.8rem;
	margin-top: 1.8rem;
}
h3 {
	font-size: 3rem;
	line-height: 1.3;
	letter-spacing: -0.1rem;
	margin-bottom: 1.5rem;
	margin-top: 1.5rem;
}
h4 {
	font-size: 2.6rem;
	line-height: 1.35;
	letter-spacing: -0.08rem;
	margin-bottom: 1.2rem;
	margin-top: 1.2rem;
}
h5 {
	font-size: 2.2rem;
	line-height: 1.5;
	letter-spacing: -0.05rem;
	margin-bottom: 0.6rem;
	margin-top: 0.6rem;
}
h6 {
	font-size: 2rem;
	line-height: 1.6;
	letter-spacing: 0;
	margin-bottom: 0.75rem;
	margin-top: 0.75rem;
}

/* Buttons
–––––––––––––––––––––––––––––––––––––––––––––––––– */
.button,
button,
input[type="submit"],
input[type="reset"],
input[type="button"] {
	display: inline-block;
	height: 38px;
	text-align: center;
	font-size: 11px;
	font-weight: 600;
	line-height: 38px;
	letter-spacing: 0.1rem;
	text-transform: uppercase;
	text-decoration: none;
	white-space: nowrap;
	cursor: pointer;
	box-sizing: border-box;
}

.button:hover,
button:hover,
input[type="submit"]:hover,
input[type="reset"]:hover,
input[type="button"]:hover,
.button:focus,
button:focus,
input[type="submit"]:focus,
input[type="reset"]:focus,
input[type="button"]:focus {
	color: #eee;
	border-color: #888;
	outline: 0;
}

.button.button-primary,
button.button-primary,
input[type="submit"].button-primary,
input[type="reset"].button-primary,
input[type="button"].button-primary {
	color: #fff;
	background-color: #33c3f0;
	border-color: #33c3f0;
}

.button.button-primary:hover,
button.button-primary:hover,
input[type="submit"].button-primary:hover,
input[type="reset"].button-primary:hover,
input[type="button"].button-primary:hover,
.button.button-primary:focus,
button.button-primary:focus,
input[type="submit"].button-primary:focus,
input[type="reset"].button-primary:focus,
input[type="button"].button-primary:focus {
	color: #fff;
	background-color: #1eaedb;
	border-color: #1eaedb;
}
